Struct Collector

struct Collector { ... }

An epoch-based garbage collector.

Implementations

impl Collector

fn new() -> Self

Creates a new collector.

fn register(self: &Self) -> LocalHandle

Registers a new handle for the collector.

impl Clone for Collector

fn clone(self: &Self) -> Self

Creates another reference to the same garbage collector.

impl Debug for Collector

fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result

impl Default for Collector

fn default() -> Self

impl Eq for Collector

impl Freeze for Collector

impl PartialEq for Collector

fn eq(self: &Self, rhs: &Collector) -> bool

Checks if both handles point to the same collector.

impl RefUnwindSafe for Collector

impl Send for Collector

impl Sync for Collector

impl Unpin for Collector

impl UnsafeUnpin for Collector

impl UnwindSafe for Collector

impl<T> Any for Collector

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for Collector

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for Collector

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> CloneToUninit for Collector

unsafe fn clone_to_uninit(self: &Self, dest: *mut u8)

impl<T> From for Collector

fn from(t: T) -> T

Returns the argument unchanged.

impl<T> Pointable for Collector

unsafe fn init(init: <T as Pointable>::Init) -> usize
unsafe fn deref<'a>(ptr: usize) -> &'a T
unsafe fn deref_mut<'a>(ptr: usize) -> &'a mut T
unsafe fn drop(ptr: usize)

impl<T> ToOwned for Collector

fn to_owned(self: &Self) -> T
fn clone_into(self: &Self, target: &mut T)

impl<T, U> Into for Collector

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for Collector

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for Collector

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>