Struct CompareExchangeError
struct CompareExchangeError<'g, T: ?Sized + Pointable, P: Pointer<T>> { ... }
The error returned on failed compare-and-swap operation.
Fields
current: Shared<'g, T>The value in the atomic pointer at the time of the failed operation.
new: PThe new value, which the operation failed to store.
Implementations
impl<'g, T, P> Freeze for CompareExchangeError<'g, T, P>
impl<'g, T, P> RefUnwindSafe for CompareExchangeError<'g, T, P>
impl<'g, T, P> Send for CompareExchangeError<'g, T, P>
impl<'g, T, P> Sync for CompareExchangeError<'g, T, P>
impl<'g, T, P> Unpin for CompareExchangeError<'g, T, P>
impl<'g, T, P> UnsafeUnpin for CompareExchangeError<'g, T, P>
impl<'g, T, P> UnwindSafe for CompareExchangeError<'g, T, P>
impl<T> Any for CompareExchangeError<'g, T, P>
fn type_id(self: &Self) -> TypeId
impl<T> Borrow for CompareExchangeError<'g, T, P>
fn borrow(self: &Self) -> &T
impl<T> BorrowMut for CompareExchangeError<'g, T, P>
fn borrow_mut(self: &mut Self) -> &mut T
impl<T> From for CompareExchangeError<'g, T, P>
fn from(t: T) -> TReturns the argument unchanged.
impl<T> Pointable for CompareExchangeError<'g, T, P>
unsafe fn init(init: <T as Pointable>::Init) -> usizeunsafe fn deref<'a>(ptr: usize) -> &'a Tunsafe fn deref_mut<'a>(ptr: usize) -> &'a mut Tunsafe fn drop(ptr: usize)
impl<T, P: Pointer<T> + fmt::Debug> Debug for CompareExchangeError<'_, T, P>
fn fmt(self: &Self, f: &mut Formatter<'_>) -> Result
impl<T, U> Into for CompareExchangeError<'g, T, P>
fn into(self: Self) -> UCalls
U::from(self).That is, this conversion is whatever the implementation of
[From]<T> for Uchooses to do.
impl<T, U> TryFrom for CompareExchangeError<'g, T, P>
fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>
impl<T, U> TryInto for CompareExchangeError<'g, T, P>
fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>